//=============================================
// J A V A  C O M M U N I T Y  P R O C E S S
// 
// J S R  3 3 1
// 
// Common Implementation
// 
//============================================= 
package javax.constraints.impl.search;

/**
 * This is a simple implementation of the SolutionIterator interface 
 * that allows a user to search and to iterate through multiple solutions.
 * The expected use:
 * <br>
 * <pre>
 * SolutionIterator iter = solver.solutionIterator();
 * while(iter.hasNext()) {
 *   Solution solution = iter.next();
 *     ...
 * }
 *
 * </pre>
 */

import javax.constraints.Solution;
import javax.constraints.SolutionIterator;
import javax.constraints.Solver;
import javax.constraints.Var;
import javax.constraints.ProblemState;
import javax.constraints.impl.constraint.ConstraintNotAllEqual;

public class BasicSolutionIterator implements SolutionIterator {
	
	Solver solver;
	Solution solution;
	int solutionNumber;
	boolean noSolutions;
	
	public BasicSolutionIterator(Solver solver) {
		this.solver = solver;
		solution = null;
		noSolutions = false;
		solutionNumber = 0;
	}

	@Override
	public boolean hasNext() {
		if (noSolutions)
			return false;
		solution = solver.findSolution(ProblemState.RESTORE);
		if (solution == null)
			return false;
		else 
			return true;
	}

	@Override
	public Solution next() {
		if (solution == null)
			throw new RuntimeException("Cannot use SolutionIterator.next() before checking the hasNext() returned true");
		solution.setSolutionNumber(++solutionNumber);
		Var[] vars = solver.getSearchStrategy().getVars();
		int[] values = new int[vars.length];
		for (int i = 0; i < values.length; i++) {
			values[i] = solution.getValue(vars[i].getName());
		}
		try {
			new ConstraintNotAllEqual(vars, values).post();
		} catch (Exception e) {
			noSolutions = true;
		}
		return solution;
	}

}
